searchData={"items":[{"type":"module","title":"ExEssentials","doc":"","ref":"ExEssentials.html"},{"type":"module","title":"ExEssentials.Web.Plugs.RequestValidator","doc":"A `Plug` for validating request parameters using `Ecto.Changeset`.\n\nThis plug enables modular and scalable validation of incoming request parameters by delegating the validation logic to a specific module (the `:validator`). It dynamically calls a function named after the current `:phoenix_action`, passing the request `params` as an argument. The function must return an `%Ecto.Changeset{}`.\n\nIf the changeset is valid, the request proceeds as usual. If invalid, a `400 Bad Request` response is sent in the `application/problem+json` format.","ref":"ExEssentials.Web.Plugs.RequestValidator.html"},{"type":"module","title":"Options - ExEssentials.Web.Plugs.RequestValidator","doc":"* `:validator` – **(required)** The module responsible for validation. It must implement a function for each action, with the same name as the `:phoenix_action` and an arity of 1 (accepting the `params`).","ref":"ExEssentials.Web.Plugs.RequestValidator.html#module-options"},{"type":"module","title":"Example Usage - ExEssentials.Web.Plugs.RequestValidator","doc":"Given a controller with the `:create` action and a validator module:\n\n    defmodule MyAppWeb.Validators.Types.User.Preferences do\n      use Ecto.Schema\n\n      import Ecto.Changeset\n\n      @themes ~w(light dark)\n      @languages ~w(en pt-br)\n      @fields ~w(theme language)a\n\n      @primary_key false\n      embedded_schema do\n        field(:theme, :string)\n        field(:language, :string)\n      end\n\n      def changeset(schema, params) do\n        schema\n        |> cast(params, @fields)\n        |> validate_required(@fields)\n        |> validate_inclusion(:theme, @themes)\n        |> validate_inclusion(:language, @languages)\n      end\n    end\n\n    defmodule MyAppWeb.Validators.Types.User.Course do\n      use Ecto.Schema\n\n      import Ecto.Changeset\n\n      @fields ~w(name duration level)a\n      @levels ~w(beginner intermediary advanced)\n\n      @primary_key false\n      embedded_schema do\n        field(:name, :string)\n        field(:duration, :integer)\n        field(:level, :string)\n      end\n\n      def changeset(schema, params) do\n        schema\n        |> cast(params, @fields)\n        |> validate_required(@fields)\n        |> validate_number(:duration, greater_than: 0)\n        |> validate_inclusion(:level, @levels)\n      end\n    end\n\n    defmodule MyAppWeb.Validators.User do\n      use Ecto.Schema\n\n      import Ecto.Changeset\n\n      alias MyAppWeb.Validators.Types.User.Course\n      alias MyAppWeb.Validators.Types.User.Preferences\n      alias MyAppWeb.Validators.User\n\n      @fields ~w(name email)a\n\n      schema \"users\" do\n        field(:name, :string)\n        field(:email, :string)\n\n        embeds_one(:preferences, Preferences)\n        embeds_many(:courses, Course)\n      end\n\n      def create(params) do\n        %User{}\n        |> cast(params, @fields)\n        |> validate_required(@fields)\n        |> cast_embed(:preferences, required: true)\n        |> cast_embed(:courses, required: true)\n      end\n    end\n\nYou can plug the validator into your controller or router pipeline:\n\n    plug ExEssentials.Web.Plugs.RequestValidator, validator: MyAppWeb.Validators.User","ref":"ExEssentials.Web.Plugs.RequestValidator.html#module-example-usage"},{"type":"module","title":"Error Response Format - ExEssentials.Web.Plugs.RequestValidator","doc":"When validation fails, the response follows the structure inspired by\n[RFC 7807 - Problem Details for HTTP APIs](https://datatracker.ietf.org/doc/html/rfc7807):\n\n    {\n      \"errors\": [\n        {\n          \"code\": \"invalid_parameter\",\n          \"detail\": \"The 'name' field can't be blank\",\n          \"title\": \"Invalid request parameters\"\n        },\n        {\n          \"code\": \"invalid_parameter\",\n          \"detail\": \"The 'courses.[1].level' field is invalid\",\n          \"title\": \"Invalid request parameters\"\n        },\n        {\n          \"code\": \"invalid_parameter\",\n          \"detail\": \"The 'preferences.language' field can't be blank\",\n          \"title\": \"Invalid request parameters\"\n        },\n        {\n          \"code\": \"invalid_parameter\",\n          \"detail\": \"The 'preferences.theme' field is invalid\",\n          \"title\": \"Invalid request parameters\"\n        }\n      ],\n      \"status_code\": 400\n    }\n\n\nEach validation error includes:\n  * `code` – configurable error code.\n  * `detail` – human-readable description of the error.\n  * `title` – general error title.","ref":"ExEssentials.Web.Plugs.RequestValidator.html#module-error-response-format"},{"type":"module","title":"Configuration - ExEssentials.Web.Plugs.RequestValidator","doc":"You can customize certain aspects via your `config.exs`:\n\n    config :ex_essentials, :web_request_validator,\n      json_library: Jason,\n      error_code: :invalid_parameter,\n      error_title: \"Invalid request parameters\"\n\nBy default, the module uses Jason to encode JSON responses, assigning the\nerror code :invalid_parameter and the error title \"Invalid request parameters\".","ref":"ExEssentials.Web.Plugs.RequestValidator.html#module-configuration"},{"type":"module","title":"Notes - ExEssentials.Web.Plugs.RequestValidator","doc":"* This plug assumes Phoenix is being used and requires `:phoenix_action` to be set in `conn.private`.\n  * `Ecto` is required for working with changesets.","ref":"ExEssentials.Web.Plugs.RequestValidator.html#module-notes"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.1"}}